@@ -134,12 +134,20 @@ doesn't exactly work well either!
 * To recover the msg, func, and line
 
 */
+#define BOTAN_FFI_SUCCESS (0)
+
+#define BOTAN_FFI_ERROR_INVALID_INPUT (-1)
+#define BOTAN_FFI_ERROR_BAD_MAC (-2)
+
 #define BOTAN_FFI_ERROR_INSUFFICIENT_BUFFER_SPACE (-10)
 #define BOTAN_FFI_ERROR_EXCEPTION_THROWN (-20)
 #define BOTAN_FFI_ERROR_BAD_FLAG (-30)
 #define BOTAN_FFI_ERROR_NULL_POINTER (-31)
+#define BOTAN_FFI_ERROR_BAD_PARAMETER (-32)
 #define BOTAN_FFI_ERROR_NOT_IMPLEMENTED (-40)
 
+#define BOTAN_FFI_ERROR_UNKNOWN_ERROR (-100)
+
 //const char* botan_error_description(int err);
 
 /**
@@ -147,6 +155,12 @@ doesn't exactly work well either!
 */
 BOTAN_DLL int botan_same_mem(const uint8_t* x, const uint8_t* y, size_t len);
 
+/**
+* Clear out memory using a system specific approach to bypass elision by the
+* compiler (currently using RtlSecureZeroMemory or tricks with volatile pointers).
+*/
+BOTAN_DLL int botan_scrub_mem(uint8_t* mem, size_t bytes);
+
 #define BOTAN_FFI_HEX_LOWER_CASE 1
 
 /**
@@ -175,7 +189,7 @@ typedef struct botan_rng_struct* botan_rng_t;
 *    "system": System_RNG, "user": AutoSeeded_RNG
 * Set rng_type to null or empty string to let the library choose
 *
-* TODO: replace rng_type with simple flags? 
+* TODO: replace rng_type with simple flags?
 */
 BOTAN_DLL int botan_rng_init(botan_rng_t* rng, const char* rng_type);
 
@@ -224,6 +238,14 @@ typedef struct botan_hash_struct* botan_hash_t;
 */
 BOTAN_DLL int botan_hash_init(botan_hash_t* hash, const char* hash_name, uint32_t flags);
 
+/**
+* Copy the state of a hash function object
+* @param dest destination hash object
+* @param source source hash object
+* @return 0 on success, a negative value on failure
+*/
+BOTAN_DLL int botan_hash_copy_state(botan_hash_t *dest, const botan_hash_t source);
+
 /**
 * Writes the output length of the hash function to *output_length
 * @param hash hash object
@@ -232,6 +254,14 @@ BOTAN_DLL int botan_hash_init(botan_hash_t* hash, const char* hash_name, uint32_
 */
 BOTAN_DLL int botan_hash_output_length(botan_hash_t hash, size_t* output_length);
 
+/**
+* Writes the block size of the hash function to *block_size
+* @param hash hash object
+* @param block_size output buffer to hold the hash function output length
+* @return 0 on success, a negative value on failure
+*/
+BOTAN_DLL int botan_hash_block_size(botan_hash_t hash, size_t* block_size);
+
 /**
 * Send more input to the hash function
 * @param hash hash object
@@ -529,12 +559,14 @@ BOTAN_DLL int botan_mp_from_bin(const botan_mp_t mp, const uint8_t vec[], size_t
 BOTAN_DLL int botan_mp_to_uint32(const botan_mp_t mp, uint32_t* val);
 
 /**
-* Return true iff mp is greater than 0
+* This function is not well named. Returns 1 iff mp is greater than
+* *or equal to* zero. Use botan_mp_is_negative to detect negative
+* numbers, botan_mp_is_zero to check for zero.
 */
 BOTAN_DLL int botan_mp_is_positive(const botan_mp_t mp);
 
 /**
-* Return true iff mp is less than 0
+* Return 1 iff mp is less than 0
 */
 BOTAN_DLL int botan_mp_is_negative(const botan_mp_t mp);
 
@@ -745,7 +777,7 @@ BOTAN_DLL int botan_privkey_get_field(botan_mp_t output,
 BOTAN_DLL int botan_privkey_load_rsa(botan_privkey_t* key,
                                      botan_mp_t p,
                                      botan_mp_t q,
-                                     botan_mp_t d);
+                                     botan_mp_t e);
 
 BOTAN_DLL int botan_privkey_rsa_get_p(botan_mp_t p, botan_privkey_t rsa_key);
 BOTAN_DLL int botan_privkey_rsa_get_q(botan_mp_t q, botan_privkey_t rsa_key);
@@ -782,6 +814,100 @@ BOTAN_DLL int botan_pubkey_dsa_get_q(botan_mp_t q, botan_pubkey_t key);
 BOTAN_DLL int botan_pubkey_dsa_get_g(botan_mp_t d, botan_pubkey_t key);
 BOTAN_DLL int botan_pubkey_dsa_get_y(botan_mp_t y, botan_pubkey_t key);
 
+/*
+* Algorithm specific key operations: ElGamal
+*/
+
+/*
+* Loads ElGamal public key
+* @param key variable populated with key material
+* @param p prime order of a Z_p group
+* @param g group generator
+* @param y public key
+*
+* @pre key is NULL on input
+* @post function allocates memory and assigns to `key'
+*
+* @return 0 on success, a negative value on failure
+*/
+BOTAN_DLL int botan_pubkey_load_elgamal(botan_pubkey_t* key,
+                                        botan_mp_t p,
+                                        botan_mp_t g,
+                                        botan_mp_t y);
+
+/*
+* Loads ElGamal private key
+*
+* @param key variable populated with key material
+* @param p prime order of a Z_p group
+* @param g group generator
+* @param x private key
+*
+* @pre key is NULL on input
+* @post function allocates memory and assigns to `key'
+*
+* @return 0 on success, a negative value on failure
+*/
+BOTAN_DLL int botan_privkey_load_elgamal(botan_privkey_t* key,
+                                         botan_mp_t p,
+                                         botan_mp_t g,
+                                         botan_mp_t x);
+
+/*
+* Algorithm specific key operations: Ed25519
+*/
+
+BOTAN_DLL int botan_privkey_load_ed25519(botan_privkey_t* key,
+                                         const uint8_t privkey[32]);
+
+BOTAN_DLL int botan_pubkey_load_ed25519(botan_pubkey_t* key,
+                                        const uint8_t pubkey[32]);
+
+BOTAN_DLL int botan_privkey_ed25519_get_privkey(botan_privkey_t key,
+                                                uint8_t output[64]);
+
+BOTAN_DLL int botan_pubkey_ed25519_get_pubkey(botan_pubkey_t key,
+                                              uint8_t pubkey[32]);
+
+/*
+* Algorithm specific key operations: ECDSA and ECDH
+*/
+BOTAN_DLL int botan_privkey_load_ecdsa(botan_privkey_t* key,
+                                       const botan_mp_t scalar,
+                                       const char* curve_name);
+
+BOTAN_DLL int botan_pubkey_load_ecdsa(botan_pubkey_t* key,
+                                      const botan_mp_t public_x,
+                                      const botan_mp_t public_y,
+                                      const char* curve_name);
+
+BOTAN_DLL int botan_pubkey_load_ecdh(botan_pubkey_t* key,
+                                     const botan_mp_t public_x,
+                                     const botan_mp_t public_y,
+                                     const char* curve_name);
+
+BOTAN_DLL int botan_privkey_load_ecdh(botan_privkey_t* key,
+                                      const botan_mp_t scalar,
+                                      const char* curve_name);
+
+BOTAN_DLL int botan_pubkey_load_sm2(botan_pubkey_t* key,
+                                    const botan_mp_t public_x,
+                                    const botan_mp_t public_y,
+                                    const char* curve_name);
+
+BOTAN_DLL int botan_privkey_load_sm2(botan_privkey_t* key,
+                                     const botan_mp_t scalar,
+                                     const char* curve_name);
+
+BOTAN_DLL int botan_pubkey_load_sm2_enc(botan_pubkey_t* key,
+                                        const botan_mp_t public_x,
+                                        const botan_mp_t public_y,
+                                        const char* curve_name);
+
+BOTAN_DLL int botan_privkey_load_sm2_enc(botan_privkey_t* key,
+                                         const botan_mp_t scalar,
+                                         const char* curve_name);
+
 /*
 * Public Key Encryption
 */
@@ -812,7 +938,7 @@ BOTAN_DLL int botan_pk_op_decrypt_destroy(botan_pk_op_decrypt_t op);
 
 BOTAN_DLL int botan_pk_op_decrypt(botan_pk_op_decrypt_t op,
                                   uint8_t out[], size_t* out_len,
-                                  uint8_t ciphertext[], size_t ciphertext_len);
+                                  const uint8_t ciphertext[], size_t ciphertext_len);
 
 /*
 * Signature Generation
@@ -862,6 +988,8 @@ BOTAN_DLL int botan_pk_op_key_agreement(botan_pk_op_ka_t op,
                                         const uint8_t other_key[], size_t other_key_len,
                                         const uint8_t salt[], size_t salt_len);
 
+BOTAN_DLL int botan_pkcs_hash_id(const char* hash_name, uint8_t pkcs_id[], size_t* pkcs_id_len);
+
 
 /*
 *
@@ -904,9 +1032,6 @@ BOTAN_DLL int botan_x509_cert_get_serial_number(botan_x509_cert_t cert, uint8_t
 BOTAN_DLL int botan_x509_cert_get_authority_key_id(botan_x509_cert_t cert, uint8_t out[], size_t* out_len);
 BOTAN_DLL int botan_x509_cert_get_subject_key_id(botan_x509_cert_t cert, uint8_t out[], size_t* out_len);
 
-BOTAN_DLL int botan_x509_cert_path_verify(botan_x509_cert_t cert,
-                                          const char* ca_dir);
-
 BOTAN_DLL int botan_x509_cert_get_public_key_bits(botan_x509_cert_t cert,
                                                   uint8_t out[], size_t* out_len);
 
@@ -938,6 +1063,17 @@ enum botan_x509_cert_key_constraints {
 
 BOTAN_DLL int botan_x509_cert_allowed_usage(botan_x509_cert_t cert, unsigned int key_usage);
 
+/**
+ * Key wrapping as per RFC 3394
+ */
+BOTAN_DLL int botan_key_wrap3394(const uint8_t key[], size_t key_len,
+                                 const uint8_t kek[], size_t kek_len,
+                                 uint8_t wrapped_key[], size_t *wrapped_key_len);
+
+BOTAN_DLL int botan_key_unwrap3394(const uint8_t wrapped_key[], size_t wrapped_key_len,
+                                   const uint8_t kek[], size_t kek_len,
+                                   uint8_t key[], size_t *key_len);
+
 /*
 * TLS (WIP)
 */
